//------------------------------------------------------------------------------
// Wolfenstein - Enemy Territory Launcher (ETL)
//------------------------------------------------------------------------------
//
//
//------------------------------------------------------------------------------
// Features :
// - Real use of autoexec.cfg (backup of old etconfig.cfg file to etconfig.etl).
//------------------------------------------------------------------------------

#include <windows.h>
#include <limits.h>

#define     ETL             "ET Launcher 0.1"
#define     WM_TRAY         WM_USER+1
#define     ETMOD           "cgame_mp_x86.dll"
#define     ETLAPP          "etl.exe"
#define     ETAPP           "et.exe"

#define     ID_TRAY         5000
#define     ID_TRAY_EXIT    5001
#define     ID_TRAY_OPT1    5002
#define     ID_TRAY_OPT2    5003


LRESULT CALLBACK WindowProcedure (HWND, UINT, WPARAM, LPARAM);
void TrayMessageHandler(HWND hWnd, UINT uMessageID);
void ShowPopupMenu(HWND hWnd);
BOOL GetEtPath (void);
BOOL EtConfigCleanup (void);
void StartET (LPSTR cmdline , LPSTR option);
void WaitFor(HANDLE processHandle);

char szClassName[ ] = "ETL";                                // Classe de l'application
HINSTANCE AppInstance = NULL;                               // Instance application
HICON g_hIcon;                                              // Icon du tray
HMENU g_hMenu;				                                // Handle to tray popup menu
HKEY  EtKey;                                                // Entre ET dans la base de registre
NOTIFYICONDATA g_iconData;	                                // Tray icon structure
char EtPath[PATH_MAX+1];                                    // Repertoire ET
BOOL  Indirect = FALSE;                                     // Appel indirect d'ET (depuis ASE par exemple)
BOOL  IsStart  = FALSE;                                     // TRUE si ET est lanc
HWND  AppHwnd;                                              // Handle application
char  CmdLine[MAX_PATH+1];                                  // Argument pass en appel direct

//------------------------------------------------------------------------------
// Point d'entre
//------------------------------------------------------------------------------
int WINAPI WinMain (HINSTANCE hThisInstance, HINSTANCE hPrevInstance, LPSTR lpszArgument, int nFunsterStil)
{
    HWND hwnd;                                              /* This is the handle for our window */
    MSG messages;                                           /* Here messages to the application are saved */
    WNDCLASSEX wincl;                                       /* Data structure for the windowclass */

    // Diver init
    IsStart = FALSE;

    /* The Window structure */
    AppInstance         = hThisInstance;
    wincl.hInstance     = hThisInstance;
    wincl.lpszClassName = szClassName;
    wincl.lpfnWndProc   = WindowProcedure;                  /* This function is called by windows */
    wincl.style         = CS_DBLCLKS;                       /* Catch double-clicks */
    wincl.cbSize        = sizeof (WNDCLASSEX);

    /* Use default icon and mouse-pointer */
    wincl.hIcon         = LoadIcon   (NULL, "BIGICON");
	wincl.hIconSm       = LoadIcon   (wincl.hInstance, "SMAICON");
    wincl.hCursor       = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName  = NULL;                             /* No menu */
    wincl.cbClsExtra    = 0;                                /* No extra bytes after the window class */
    wincl.cbWndExtra    = 0;                                /* structure or the window instance */
    wincl.hbrBackground = (HBRUSH) COLOR_BACKGROUND;        /* Use Windows's default color as the background of the window */
	g_hIcon             = wincl.hIconSm;

    // Verification ET
//    if (!GetEtPath ())
//        return 0;

    /* Register the window class, and if it fails quit the program */
    if (!RegisterClassEx (&wincl))
        return 0;

    /* The class is registered, let's create the program*/
    hwnd = CreateWindowEx (0, szClassName, ETL, WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, 0 , 0, 0, NULL, NULL, hThisInstance, NULL);
    AppHwnd = hwnd;

    // Creation tray icon
	g_iconData.cbSize              = sizeof(NOTIFYICONDATA);
	g_iconData.hIcon               = g_hIcon;
	g_iconData.hWnd                = hwnd;
	strcpy(g_iconData.szTip, "ET Laucher");
	g_iconData.uCallbackMessage    = WM_TRAY;
	g_iconData.uFlags              = NIF_ICON | NIF_MESSAGE | NIF_TIP;
	g_iconData.uID                 = ID_TRAY;
	Shell_NotifyIcon(NIM_ADD, &g_iconData);

    // Creation du menu pop up tray
	g_hMenu = CreatePopupMenu();
	AppendMenu(g_hMenu, MF_STRING, ID_TRAY_OPT1, "Essai 1");
	AppendMenu(g_hMenu, MF_SEPARATOR, 0, 0);
	AppendMenu(g_hMenu, MF_STRING, ID_TRAY_EXIT, "Quit");

    // Lancement de ET si appell avec argument ligne de commande
    strcpy (CmdLine, lpszArgument);
    if (strlen(lpszArgument) != 0)
        {
        Indirect = TRUE;
        StartET (CmdLine, "");
        PostMessage (hwnd, WM_DESTROY, 0, 0);
        }

    /* Run the message loop. It will run until GetMessage() returns 0 */
    while (GetMessage (&messages, NULL, 0, 0))
        {
        /* Translate virtual-key messages into character messages */
        TranslateMessage(&messages);
        /* Send message to WindowProcedure */
        DispatchMessage(&messages);
        }
    
    return messages.wParam;
}


//------------------------------------------------------------------------------
// Fonction appele par DispatchMessage
//------------------------------------------------------------------------------
LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)                  /* handle the messages */
        {
        // Quit de l'application
        case WM_DESTROY:
    		g_iconData.uFlags = 0;
	       	Shell_NotifyIcon(NIM_DELETE,&g_iconData);
            PostQuitMessage (0);       /* send a WM_QUIT to the message queue */
            break;

        // Gestion des messages pour la tray bar
        case WM_TRAY:
            // Right click - Affichage du menu
            if (WM_RBUTTONDOWN == lParam)
	            ShowPopupMenu(hwnd);
            // Mode config
//            else if (WM_LBUTTONDBLCLK == lParam)
//            	TrayMessageHandler(hwnd, ID_TRAY_CONFIG);
            break;

        case WM_COMMAND:
        	TrayMessageHandler(hwnd, LOWORD(wParam));            
            break;

        default:                      /* for messages that we don't deal with */
            return DefWindowProc (hwnd, message, wParam, lParam);
        }

    return 0;
}

//------------------------------------------------------------------------------
// Gestion des messages pour la tray 
//------------------------------------------------------------------------------
void TrayMessageHandler(HWND hWnd, UINT uMessageID)
{
    int nIndex = 0;

	switch (uMessageID)
    	{
        // Sortie du programme
        case ID_TRAY_EXIT:          
            DestroyWindow(hWnd);
            break;

        // Appel de la configuration
        case ID_TRAY_OPT1:
            SetForegroundWindow(hWnd);
            InsertMenu(g_hMenu, nIndex, MF_BYPOSITION | MF_STRING, nIndex + 1, "heelo");                                                
            EtConfigCleanup ();
            break;
        }
}
//------------------------------------------------------------------------------
// Affichage du menu principal du programme (tray) 
//------------------------------------------------------------------------------
void ShowPopupMenu(HWND hWnd)
{
	POINT pt;
 	
	// Affichage du pop up a la position courante
	GetCursorPos(&pt);
	SetForegroundWindow(hWnd);	
	TrackPopupMenu(g_hMenu, 0, pt.x, pt.y, 0, hWnd, 0);	
	PostMessage(hWnd, WM_NULL, 0, 0);
}

//------------------------------------------------------------------------------
// Start ET
//------------------------------------------------------------------------------
void StartET (LPSTR cmdline , LPSTR option)
{
    char buf[MAX_PATH+1];
	int res;
	STARTUPINFO si;
	PROCESS_INFORMATION pi;
    DWORD ret;
    MSG msg;

    IsStart = TRUE;
	sprintf (buf,"%s\\%s %s %s", EtPath, ETAPP, cmdline, option);
	
	memset((void *)&si,0,sizeof(STARTUPINFO));
    si.cb = sizeof(STARTUPINFO);
    res = CreateProcess(NULL,buf,NULL,NULL,0,NORMAL_PRIORITY_CLASS,NULL,NULL,&si,&pi);
    if (res)
		{
        IsStart = TRUE;
        do
            {
            ret = MsgWaitForMultipleObjects(1, &pi.hProcess, FALSE, INFINITE, QS_PAINT | QS_SENDMESSAGE);
            if (ret == WAIT_FAILED)
                break;
            if (ret == (WAIT_OBJECT_0 + 1))
                {
                while (PeekMessage(&msg, 0, 0, 0, PM_REMOVE))
                    {
                    DispatchMessage(&msg);
                    }
                }
            }
        while (ret == WAIT_OBJECT_0);

        // Fin ET
        //GetExitCodeProcess(pi.hProcess, &result);
		CloseHandle (pi.hProcess);
		CloseHandle (pi.hThread);
		}
}

//------------------------------------------------------------------------------
// Recherche du path de ET depuis la base de registre
//------------------------------------------------------------------------------
BOOL GetEtPath (void)
{
    DWORD ta=PATH_MAX,ty=REG_SZ;
    char tmp[PATH_MAX+1];

    // Recherche de la clef
    if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE\\Activision\\Wolfenstein - Enemy Territory", 0, KEY_ALL_ACCESS, &EtKey) == ERROR_SUCCESS)
        {
        if(RegQueryValueEx(EtKey, "InstallPath", 0, &ty, EtPath, &ta) != ERROR_SUCCESS)
            {
            MessageBox (NULL, "'InstallPath' key for Enemy Territory not found", ETL, MB_ICONERROR);
            RegCloseKey(EtKey);
            return FALSE;
            }
        }
    else
        {
        MessageBox (NULL, "Enemy Territory key not found", ETL, MB_ICONERROR);
        RegCloseKey(EtKey);
        return FALSE;
        }
    RegCloseKey(EtKey);

    // Check du bon emplacement de ETL
    sprintf (tmp, "%s\\%s", EtPath, ETLAPP);
    if (GetFileAttributes (tmp) == 0xFFFFFFFF)
        {
        MessageBox (NULL, "ETL must be in the base path of Enemy Territory (same as ET.exe)", ETL, MB_ICONERROR);
        return FALSE;
        }

    return TRUE;
}

//------------------------------------------------------------------------------
// Suppression des etconfig.cfg et profile.pid inutile                  
//------------------------------------------------------------------------------
BOOL EtConfigCleanup (void)
{
    WIN32_FIND_DATA FindFileData,ff;
	HANDLE hFind,hf;
    TCHAR apath[MAX_PATH+1],base[MAX_PATH+1], amod[MAX_PATH+1], tmp[MAX_PATH+1];
    int nIndex = 0, ln;

    // Preparation
	sprintf (base, "%s\\*.*", EtPath);
	if ((hFind = FindFirstFile(base, &FindFileData)) == INVALID_HANDLE_VALUE)
        {
        MessageBox (NULL, "Enemy Territory path not found", ETL, MB_ICONERROR);
        return FALSE;
        }

    // Parcours du repertoire ET
	do
		{
        // Seul les repertoires interressent
		if (FindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			{
			if (strcmp (".", FindFileData.cFileName) == 0)
                continue;
			if (strcmp ("..", FindFileData.cFileName) == 0)					
                continue;

            // Sur repertoire trouv, test d'un mod
            sprintf (apath, "%s\\%s\\%s", EtPath, FindFileData.cFileName, ETMOD);				
            if (GetFileAttributes (apath) == 0xFFFFFFFF)
                continue;

            // On est dans un repertoire de mod que l'on va parcourir pour les profils
            strncpy (amod, FindFileData.cFileName, MAX_PATH);
            sprintf (base,"%s\\%s\\profiles\\*.*", EtPath, amod);

    		if ((hf = FindFirstFile(base, &ff)) != INVALID_HANDLE_VALUE)
                {
                do
                    {
                    // Seule les repertoires sont des profiles
                    if (ff.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
		        	    {
                        if (strcmp (".", ff.cFileName) != 0)
                            {
            		  	    if (strcmp ("..", ff.cFileName) != 0)					
                                {
                                // Suppression etconfig
        				        sprintf (apath, "%s\\%s\\profiles\\%s\\etconfig.cfg", EtPath, amod, ff.cFileName);
                                if (GetFileAttributes (apath) != 0xFFFFFFFF)
                                    {
                                    // Check existence d'un backup et backup si inexistant
                                    strcpy (tmp, apath);
                                    ln = strlen (tmp);
                                    tmp[ln-3] = 'e' ; tmp[ln-2] = 't' ; tmp[ln-1] = 'l';
                                    InsertMenu(g_hMenu, nIndex, MF_BYPOSITION | MF_STRING, nIndex + 1, tmp);                                                
                                    CopyFile (apath, tmp, TRUE);
            						remove (apath);		
                                    }				                    	
            					// Suppression profile.pid
    		          		    sprintf (apath, "%s\\%s\\profiles\\%s\\profile.pid", EtPath, amod, ff.cFileName);						
                                if (GetFileAttributes (apath) != 0xFFFFFFFF)
            						remove (apath);	
	
                                } // fin controle '..'
                            } // fin controle '.'
                        } // fin FindFileData
                    } // fin Do
                while (FindNextFile(hf, &ff));
                } // fin FindFirst
                
            // 
            FindClose (hf);
			}
	    }
	while (FindNextFile(hFind, &FindFileData));
   
	// Fin et sortie
	FindClose(hFind);
	return TRUE;
}


